#!/bin/sh
#
#       @(#) exportpaths.shinc 12.5 97/11/19 
#
# Copyright (c) 1997 The Santa Cruz Operation, Inc.. All Rights Reserved.
# THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF THE SANTA CRUZ OPERATION INC.
# The copyright notice above does not evidence any actual or intended
# publication of such source code.
# 

##########################################################################
#
# shell script to implement symlinks for Netscape products
# reads l_info and link.it and links the required file versions
# attempts localization, using either the 'exportpaths' message
# catalog or an exportpaths.msgs file in the packaging
#
# Arguments are:
#
#	PKGINST		- the package instance name (used for installf
#			  and removef)
#	links.fl_path	- path to the link information file
#	li_info_path	- path to the l_info file, for LINFO_* definitions
#	menu_path	- path to the directory containing error messages
#
# Exit codes are:
#
#	0		- success (may include WARNINGs)
#	1		- failure (on ERROR, normally)
#	other non-zero	- exit status from underlying command
#
##########################################################################

# definitions
TRUE=1; FALSE=0
SUCCESS=0; FAIL=1
ME="`basename $0 .sh`"
USAGE="Usage: %s PKGINST links.fl_path l_info_path menu_path\n"
MESSAGES_DEFS="exportpaths.msgs"
LABEL="UX:exportpaths"
CAT="exportpaths"

# included standard routines
##########################################################################
#
#	@(#) test_license.sh_h 11.5 97/10/15 
#
# Standard routine to check if a license is already covered by something
# installed on the system
#
# Arguments are:
#	$1	- y-bit flags values to test for - this is a set of
#		  y-values, separated by colons.  If any match then
#		  the product is considered licensed
#
# Return values are:
#	$SUCCESS- license is already covered by something else
#	$FAIL	- no license covers this product
#
##########################################################################
test_license()
{
	test_string="$1"

	# elements in $1 are separated by colons - test each element
	# to see if the licensing indicated by them is covered; if it
	# is, we can return $SUCCESS, else we return $FAIL
	for license_string in `echo "$test_string" | tr ' :' ': '`; do
		license_arg="`echo \"$license_string\" | tr ':' ' '`"

		# pass the args string as an argument (in its totality)
		# to brand; if brand returns zero then this license
		# check succeeded (the license is covered)
		/usr/sbin/brand $license_arg >/dev/null 2>&1
		if [ $? -eq 0 ]; then
			return $SUCCESS
		fi
	done

	# no licensing element is covered, so return $FAIL
	return $FAIL
}



# trap to ignore signals
trap '' 1 2 3 15

# arguments for directories and file definitions
if [ $# -ne 4 ]; then
	pfmt -s action -g $CAT:1 "$USAGE" "$ME"
	exit $FAIL
fi

pkginst="$1"
LINKS_FL="$2"
L_INFO_PATH="$3"
if [ "`echo \"$L_INFO_PATH\" | cut -c1`" != '/' ]; then
	# not absolute, so add pwd to make absolute
	L_INFO_PATH="`pwd`/$L_INFO_PATH"
fi
MENU_PATH="$2"
if [ "`echo \"$MENU_PATH\" | cut -c1`" != '/' ]; then
	# not absolute, so add pwd to make absolute
	MENU_PATH="`pwd`/$MENU_PATH"
fi

# read message definitions
if [ -f $MENU_PATH/$MESSAGES_DEFS ]; then
        . $MENU_PATH/$MESSAGES_DEFS
else
	# emergency English error messages
	MSG_MISSING_FILE="can't find file %s\n"
	MSG_NO_LINFO_ENCRYPT_ARGS="%s doesn't define LINFO_ENCRYPT_ARGS\n"
	MSG_CANT_REMOVE_LINK="unable to remove %s\n"
	MSG_TARGET_DOESNT_EXIST="%s does not exist\n"
	MSG_OVERWRITING_LINK="overwriting %s\n"
	MSG_UNABLE_TO_LINK="unable to link %s to %s\n"
	MSG_UNABLE_TO_SYMLINK="unable to link %s to %s\n"
fi


##########################################################################
#
# fatal error function
#
##########################################################################
exportpaths_fatal()
{
	# print out the fatal error message and exit
	mnum=$1; msg="$2"; shift; shift
	pfmt -l $LABEL -s error -g $CAT:$mnum "$msg" $*

	# exit
	exit $FAIL
} # exportpaths_fatal()


##########################################################################
#
# main
#
##########################################################################

# then read in the l_info file, to get the LINFO_* definitions
if [ -f "$L_INFO_PATH" ]; then
	. $L_INFO_PATH
else
	exportpaths_fatal 2 "$MSG_MISSING_FILE" "$L_INFO_PATH"
fi

# see if this package has any special encryption needs
fc="`echo \"$LINFO_NO_ENCRYPT\" | cut -c1`"
if [ "$fc" = "Y" -o "$fc" = "y" ]; then
	# no need to do anything more
	exit $SUCCESS
fi

# LINFO_ENCRYPT_ARGS needs to be part of the environment read from the file
if [ -z "$LINFO_ENCRYPT_ARGS" ]; then
	exportpaths_fatal 3 "$MSG_NO_LINFO_ENCRYPT_ARGS" "$L_INFO_PATH"
fi

# test the licensing for the encryption arguments, to see if we should
# be linking the strong encrypted or other versions; set a value for
# ENCRYPT (the first char we'd look for in the file)
test_license "$LINFO_ENCRYPT_ARGS"
if [ $? -eq $SUCCESS ]; then
	ENCRYPT="U"
else
	ENCRYPT="I"
fi

# format of data file is
# [I|U|G]:target:link
# I = International
# U = US/Domestic
# G = General (both US and International)

# install or remove files associated with current license type
if [ ! -f "$LINKS_FL" ]; then
	exportpaths_fatal 2 "$MSG_MISSING_FILE" "$LINKS_FL"
fi

# first, go through and remove all the links this file defines, if
# present.  This loop finds all the link fields (field 3), then
# sorts and uniq's them, since otherwise removef will indicate
# that second and subsequent occurrences of a filename may be
# removed.  It then uses removef as a filter to tell it which files
# may actually be removed (that is, not part of any other package)
egrep -e "^(I|U|G):" $LINKS_FL | awk -F":" '{ print $3 }' 2>/dev/null \
		| sort | uniq | removef "$pkginst" - | while read link; do

	# verified the removal is safe, then remove it
	if [ -f "$link" ]; then
		rm $link >/dev/null 2>&1
		if [ $? -ne 0 ]; then
			pfmt -l $LABEL -s warn -g $CAT:4 \
					"$MSG_CANT_REMOVE_LINK" "$link"
		fi
	fi
done

# now, go through again and add all the links we should be adding
# This time, the loop finds only the link types we are interested
# in processing, and makes the links; the loop writes the link
# information to stdout in installf format - this is piped into
# installf
egrep -e "^(${ENCRYPT}|G):" $LINKS_FL | \
		awk -F":" '{ print $1" "$2" "$3" "$4 }' | \
		while read encrypt target link type; do

	# if the target does not exist then warn and ignore
	if [ ! -f $target -a ! -L $target ]; then
		pfmt -l $LABEL -s warn -g $CAT:5 \
				"$MSG_TARGET_DOESNT_EXIST" "$target"
		continue
	fi

	# if the link already exists and is to a different target then
	# warn but continue - this allows a link to be 'shared' between
	# packages if this is necessary, provided the targets are always
	# the same in these packages (the installf/removef mechanisms
	# allow this)
	if [ -L $link ]; then
		if [ "$type" != "S" ]; then
			# old link is S, but new one isn't
			pfmt -l $LABEL -s warn -g $CAT:6 \
					"$MSG_OVERWRITING_LINK" "$link"
		else
			# link types match - check target
			curr_target="`ls -l $link | awk '{ print $NF }'`"
			if [ -n "$curr_target" -a \
					"$target" != "$curr_target" ]; then
				pfmt -l $LABEL -s warn -g $CAT:6 \
						"$MSG_OVERWRITING_LINK" "$link"
			fi
		fi
		rm $link 2>/dev/null
	else
		if [ -f $link ]; then
			if [ "$type" = "S" ]; then
				# old link isn't S, but new one is
				pfmt -l $LABEL -s warn -g $CAT:6 \
						"$MSG_OVERWRITING_LINK" "$link"
			else
				# link types match - check target
				cmp -s $link $target 2>/dev/null
				if [ $? -ne 0 ]; then
					pfmt -l $LABEL -s warn -g $CAT:6 \
						"$MSG_OVERWRITING_LINK" "$link"
				fi
			fi
			rm $link 2>/dev/null
		fi
	fi

	# the record matches our encrypt type or it is general
	# so add the new link
	if [ "$type" = "H" ]; then
		ln $target $link >/dev/null 2>&1
		if [ $? -eq 0 ]; then
			# output update package information with the link
			echo "$link=$target"
		else
			pfmt -l $LABEL -s error -g $CAT:7 \
				"$MSG_UNABLE_TO_LINK" "$link" "$target"
		fi
	else
		ln -s $target $link >/dev/null 2>&1
		if [ $? -eq 0 ]; then
			# output update package information with the link
			echo "$link=$target s"
		else
			pfmt -l $LABEL -s error -g $CAT:8 \
				"$MSG_UNABLE_TO_SYMLINK" "$link" "$target"
		fi
	fi

done | installf "$pkginst" -

removef -f "${pkginst}"
installf -f "${pkginst}"

# done
exit $SUCCESS
