#!/bin/sh
#
#	@(#) releasecheck.shinc 12.6 97/11/19 
#
# Copyright (c) 1997 The Santa Cruz Operation, Inc.. All Rights Reserved.
# THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF THE SANTA CRUZ OPERATION INC.
# The copyright notice above does not evidence any actual or intended
# publication of such source code.
#


##########################################################################
#
# shell script to check for [in]compatible UnixWare releases
#
# check the uname -v result against a list of known [in]compatilities
#
# Arguments are:
#
#	[-q]		- run without producing menus
#	menu_path	- path to the directory containing the forms
#			  and messages
#
# Exit codes are:
#
#	0		- success - no incompatibility found
#	1		- failure - the installation should be aborted
#
##########################################################################

# definitions
TRUE=1; FALSE=0
SUCCESS=0; FAIL=1
ME="`basename $0 .sh`"
USAGE="Usage: $ME menu_path"
MESSAGES_DEFS="releasecheck.msgs"
UNAME="/bin/uname"

# trap to ignore signals
trap '' 1 2 3 15

# arguments for menu definitions
QUIET=$FALSE
while getopts "q" opt; do
	case "$opt" in
	'q')	QUIET=$TRUE		;;
	'?')	echo "$USAGE" >&2
		exit $FAIL		;;
	esac
done
shift `expr $OPTIND - 1`
if [ $# -ne 1 ]; then
	echo "$USAGE" >&2
	exit $FAIL
fi

MENU_PATH="$1"
if [ "`echo \"$MENU_PATH\" | cut -c1`" != '/' ]; then
	# not absolute, so add pwd to make absolute
	MENU_PATH="`pwd`/$MENU_PATH"
fi

# set up our path
PATH=/bin:/usr/bin:/sbin:/usr/sbin:/etc
export PATH

# read message definitions; alternatives if we can't do menus
if [ -f $MENU_PATH/$MESSAGES_DEFS ]; then
	. $MENU_PATH/$MESSAGES_DEFS
else
	echo "$ME: internal error - no $MESSAGES_DEFS file found" >&2
	exit $FAIL
fi

# symbols to help find menus and menu processing
MENU_COLORS="menu_colors"
ERROR_COLOR="error"
WARNING_COLOR="warn"
REGULAR_COLOR="regular"
MENU="menu"
MENU_TMPFILE="/tmp/releasecheck.menu$$"
INCOMPATIBLE_MENU="menu.releasecheck01"
INDETERMINATE_MENU="menu.releasecheck02"
BADOS_MENU="menu.releasecheck03"

# menu stuff
LOCALE="${LC_ALL:-${LC_MESSAGES:-${LANG:-C}}}"
# set up to use menu_colors; default to C-locale if locale's dir has
# no menu_colors.sh
if [ -f /etc/inst/locale/$LOCALE/menus/menu_colors.sh ]
then
	. /etc/inst/locale/$LOCALE/menus/menu_colors.sh
else
	if [ -f /etc/inst/locale/C/menus/menu_colors.sh ]; then
		. /etc/inst/locale/C/menus/menu_colors.sh
	else
		echo "$ME: internal error - no /etc/inst/locale/C/menus/menu_colors.sh file found" >&2
		exit $FAIL
	fi
fi


##########################################################################
#
# Menu forms library function
#	$1 - name of the form to run
#	$2 - file to send form (menu) -o output to
#	$3 - color set to use (regular, warn, or error)
#	$4 - string to eval as forms arguments
#
##########################################################################
run_form()
{
	# catch function parameters
	form_name="$1"
	output_file="$2"
	color="$3"
	form_args="$4"

	# set requested menu colors
	$MENU_COLORS $color

	# find the form that we have been asked to run; complain if
	# not there
	form_path="$MENU_PATH/$form_name"
	if [ ! -f "$form_path" ]; then
		echo "$ME: internal error - no form $form_name" >&2
		exit $FAIL
	fi

	# build a form header line
	form_header="$ME`[ -n "$PKGNAME" ] && echo \" ($PKGNAME)\"`"

	# run the form and send output to $2 parameter
	eval "FORM_HEADER=\"$form_header\" $form_args \
			$MENU $form_opts -f $form_path -o $output_file"
} # run_form()


##########################################################################
#
# main
#
##########################################################################

# if we are called from pkgadd then PKG will be set to the id of the
# package, and NAME will be set to the package name - trap these so
# they can be used in forms to remind the user what exactly they are
# installing
PKGNAME=""
if [ -n "$PKG" ]; then
	PKGNAME="$NAME"
fi

# only do something if there's a UNIXWARE_INCOMPATIBLE and/or a
# UNIXWARE_COMPATIBLE environment variable (from the pkginfo file)
if [ -n "$UNIXWARE_INCOMPATIBLE" -o -n "$UNIXWARE_COMPATIBLE" ]; then

	# get the name and release number of the platform we are on
	osname="`uname -s`"
	release="`uname -v`"

	# check that this operating system is UnixWare
	if [ "$osname" != "UNIX_SV" -a "$osname" != "UnixWare" ]; then
		if [ $QUIET -ne $TRUE ]; then
			run_form "$BADOS_MENU" "$MENU_TMPFILE" \
				"$ERROR_COLOR" \
				"FORM_OS=\"$osname\" \
				FORM_RELEASE=\"$release\" \
			 	FORM_COMPATOS=\"UNIX_SV, UnixWare\""
			rm -f $MENU_TMPFILE
		else
			echo "\n\n$MSG_BADOS\n" >&2
		fi
		exit $FAIL
	fi

	# attempt a match against each of the incompatibilities
	# we are aware of
	incompatible=0
	if [ -n "$UNIXWARE_INCOMPATIBLE" ]; then
		for incompat in `echo "$UNIXWARE_INCOMPATIBLE" | \
				tr ',:' '  '`; do
			case $release in
			$incompat)	incompatible=1
					break		;;
			*)				;;
			esac
		done
	fi

	# now do the same for the compatibilities
	compatible=0
	if [ -n "$UNIXWARE_COMPATIBLE" ]; then
		for compat in `echo "$UNIXWARE_COMPATIBLE" | \
				tr ',:' '  '`; do
			case $release in
			$compat)	compatible=1
					break		;;
			*)				;;
			esac
		done
	fi

	# if determined compatible then return success
	if [ $compatible -eq 1 ]; then
		exit $SUCCESS
	fi

	# if determined incompatible then display the dialog/message and
	# return failure
	if [ $incompatible -eq 1 ]; then
		if [ $QUIET -ne $TRUE ]; then
			run_form "$INCOMPATIBLE_MENU" "$MENU_TMPFILE" \
				"$ERROR_COLOR" \
				"FORM_RELEASE=\"$release\" \
			 	FORM_INCOMPATIBLE=\"$UNIXWARE_INCOMPATIBLE\" \
			 	FORM_COMPATIBLE=\"$UNIXWARE_COMPATIBLE\""
			rm -f $MENU_TMPFILE
		else
			echo "\n\n$MSG_INCOMPATIBLE\n" >&2
		fi
		exit $FAIL
	fi

	# if neither incompatible nor compatible then allow the user
	# to choose whether to continue or fail the install
	if [ $QUIET -ne $TRUE ]; then
		run_form "$INDETERMINATE_MENU" "$MENU_TMPFILE" \
			"$WARNING_COLOR" \
			"FORM_RELEASE=\"$release\" \
		 	FORM_INCOMPATIBLE=\"$UNIXWARE_INCOMPATIBLE\" \
		 	FORM_COMPATIBLE=\"$UNIXWARE_COMPATIBLE\""
		if [ -f "$MENU_TMPFILE" ]; then
			. $MENU_TMPFILE; rm $MENU_TMPFILE
		else
			RETURN_VALUE=2
		fi
		if [ $RETURN_VALUE -eq 1 ]; then
			# continue the installation
			exit $SUCCESS
		else
			# cancel the installation
			exit $FAIL
		fi
	else
		# if we must be silent then simply fail the attempt
		echo "\n\n$MSG_INCOMPATIBLE\n" >&2
		exit $FAIL
	fi
else

	# no compatibilities specified
	exit $SUCCESS
fi

# don't expect to reach here
echo "$ME: unexpected completion of script" >&2
exit $FAIL
